@@ -37,13 +37,14 @@ use std::io::{Read, Write};
 use std::path::{Path, PathBuf};
 use std::process::{self, Command};
 
-use crate::error::{Blame, MainError, Result};
-use crate::util::{ChainMap, Defer};
+use crate::error::{MainError, MainResult};
+use crate::util::Defer;
 use sha1::{Digest, Sha1};
 
 #[derive(Debug)]
 struct Args {
-    command: Vec<String>,
+    script: Option<String>,
+    script_args: Vec<String>,
     features: Option<String>,
 
     expr: bool,
@@ -53,13 +54,13 @@ struct Args {
     pkg_path: Option<String>,
     gen_pkg_only: bool,
     build_only: bool,
+    cargo_output: bool,
     clear_cache: bool,
     debug: bool,
     dep: Vec<String>,
     extern_: Vec<String>,
     force: bool,
     unstable_features: Vec<String>,
-    use_bincache: Option<bool>,
     build_kind: BuildKind,
     template: Option<String>,
     list_templates: bool,
@@ -111,77 +112,97 @@ fn parse_args() -> Args {
     let app = App::new(consts::PROGRAM_NAME)
         .version(version)
         .setting(clap::AppSettings::TrailingVarArg)
+        .setting(clap::AppSettings::AllowLeadingHyphen)
         .about(about)
-            .arg(Arg::with_name("command")
-                .help("Command (script file plus arguments) to execute.")
+            .arg(Arg::new("script")
                 .index(1)
-                .required(true)
+                .about("Script file or expression to execute.")
+                .required_unless_present_any(if cfg!(windows) {
+                    vec!["clear-cache", "list-templates", "install-file-association", "uninstall-file-association"]
+                } else {
+                    vec!["clear-cache", "list-templates"]
+                })
+                .number_of_values(1)
+                .conflicts_with_all(if cfg!(windows) {
+                    &["list-templates", "install-file-association", "uninstall-file-association"]
+                } else {
+                    &["list-templates"]
+                })
+            )
+            .arg(Arg::new("script-args")
+                .index(2)
+                .about("Arguments for the script to execute.")
                 .multiple(true)
-                .conflicts_with_all(&["list-templates", "file-association"])
+                .min_values(0)
             )
-            .arg(Arg::with_name("expr")
-                .help("Execute <script> as a literal expression and display the result.")
+            .arg(Arg::new("expr")
+                .about("Execute <script> as a literal expression and display the result.")
                 .long("expr")
-                .short("e")
+                .short('e')
                 .takes_value(false)
-                .requires("command")
+                .requires("script")
             )
-            .arg(Arg::with_name("loop")
-                .help("Execute <script> as a literal closure once for each line from stdin.")
+            .arg(Arg::new("loop")
+                .about("Execute <script> as a literal closure once for each line from stdin.")
                 .long("loop")
-                .short("l")
+                .short('l')
                 .takes_value(false)
-                .requires("command")
+                .requires("script")
             )
-            .group(ArgGroup::with_name("expr_or_loop")
+            .group(ArgGroup::new("expr_or_loop")
                 .args(&["expr", "loop"])
-                .conflicts_with_all(&["list-templates", "file-association"])
             )
             /*
             Options that impact the script being executed.
             */
-            .arg(Arg::with_name("count")
-                .help("Invoke the loop closure with two arguments: line, and line number.")
+            .arg(Arg::new("cargo-output")
+                .about("Show output from cargo when building.")
+                .short('o')
+                .long("cargo-output")
+                .requires("script")
+            )
+            .arg(Arg::new("count")
+                .about("Invoke the loop closure with two arguments: line, and line number.")
                 .long("count")
                 .requires("loop")
             )
-            .arg(Arg::with_name("debug")
-                .help("Build a debug executable, not an optimised one.")
+            .arg(Arg::new("debug")
+                .about("Build a debug executable, not an optimised one.")
                 .long("debug")
             )
-            .arg(Arg::with_name("dep")
-                .help("Add an additional Cargo dependency.  Each SPEC can be either just the package name (which will assume the latest version) or a full `name=version` spec.")
+            .arg(Arg::new("dep")
+                .about("Add an additional Cargo dependency. Each SPEC can be either just the package name (which will assume the latest version) or a full `name=version` spec.")
                 .long("dep")
-                .short("d")
+                .short('d')
                 .takes_value(true)
                 .multiple(true)
                 .number_of_values(1)
             )
-            .arg(Arg::with_name("dep_extern")
-                .help("Like `dep`, except that it *also* adds a `#[macro_use] extern crate name;` item for expression and loop scripts.  Note that this only works if the name of the dependency and the name of the library it generates are exactly the same.")
+            .arg(Arg::new("dep_extern")
+                .about("Like `dep`, except that it *also* adds a `#[macro_use] extern crate name;` item for expression and loop scripts. Note that this only works if the name of the dependency and the name of the library it generates are exactly the same.")
                 .long("dep-extern")
-                .short("D")
+                .short('D')
                 .takes_value(true)
                 .multiple(true)
                 .requires("expr_or_loop")
             )
-            .arg(Arg::with_name("extern")
-                .help("Adds an `#[macro_use] extern crate name;` item for expressions and loop scripts.")
+            .arg(Arg::new("extern")
+                .about("Adds an `#[macro_use] extern crate name;` item for expressions and loop scripts.")
                 .long("extern")
-                .short("x")
+                .short('x')
                 .takes_value(true)
                 .multiple(true)
                 .requires("expr_or_loop")
             )
-            .arg(Arg::with_name("features")
-                 .help("Cargo features to pass when building and running.")
+            .arg(Arg::new("features")
+                 .about("Cargo features to pass when building and running.")
                  .long("features")
                  .takes_value(true)
             )
-            .arg(Arg::with_name("unstable_features")
-                .help("Add a #![feature] declaration to the crate.")
+            .arg(Arg::new("unstable_features")
+                .about("Add a #![feature] declaration to the crate.")
                 .long("unstable-feature")
-                .short("u")
+                .short('u')
                 .takes_value(true)
                 .multiple(true)
                 .requires("expr_or_loop")
@@ -190,78 +211,71 @@ fn parse_args() -> Args {
             /*
             Options that change how cargo script itself behaves, and don't alter what the script will do.
             */
-            .arg(Arg::with_name("build_only")
-                .help("Build the script, but don't run it.")
+            .arg(Arg::new("build_only")
+                .about("Build the script, but don't run it.")
                 .long("build-only")
                 .requires("script")
-                .conflicts_with_all(&["args"])
+                .conflicts_with_all(&["script-args"])
             )
-            .arg(Arg::with_name("clear_cache")
-                .help("Clears out the script cache.")
+            .arg(Arg::new("clear-cache")
+                .about("Clears out the script cache.")
                 .long("clear-cache")
             )
-            .arg(Arg::with_name("force")
-                .help("Force the script to be rebuilt.")
+            .arg(Arg::new("force")
+                .about("Force the script to be rebuilt.")
                 .long("force")
                 .requires("script")
             )
-            .arg(Arg::with_name("gen_pkg_only")
-                .help("Generate the Cargo package, but don't compile or run it.")
+            .arg(Arg::new("gen_pkg_only")
+                .about("Generate the Cargo package, but don't compile or run it.")
                 .long("gen-pkg-only")
                 .requires("script")
-                .conflicts_with_all(&["args", "build_only", "debug", "force", "test", "bench"])
+                .conflicts_with_all(&["script-args", "build_only", "debug", "force", "test", "bench"])
             )
-            .arg(Arg::with_name("pkg_path")
-                .help("Specify where to place the generated Cargo package.")
+            .arg(Arg::new("pkg_path")
+                .about("Specify where to place the generated Cargo package.")
                 .long("pkg-path")
                 .takes_value(true)
-                .requires("command")
-                .conflicts_with_all(&["clear_cache", "force"])
+                .requires("script")
+                .conflicts_with_all(&["clear-cache", "force"])
             )
-            .arg(Arg::with_name("use_bincache")
-                .help("Override whether or not the shared binary cache will be used for compilation.")
-                .long("use-shared-binary-cache")
-                .takes_value(true)
-                .possible_values(&["no", "yes"])
-            )
-            .arg(Arg::with_name("test")
-                .help("Compile and run tests.")
+            .arg(Arg::new("test")
+                .about("Compile and run tests.")
                 .long("test")
-                .conflicts_with_all(&["bench", "debug", "args", "force"])
+                .conflicts_with_all(&["bench", "debug", "script-args", "force"])
             )
-            .arg(Arg::with_name("bench")
-                .help("Compile and run benchmarks. Requires a nightly toolchain.")
+            .arg(Arg::new("bench")
+                .about("Compile and run benchmarks. Requires a nightly toolchain.")
                 .long("bench")
-                .conflicts_with_all(&["test", "debug", "args", "force"])
+                .conflicts_with_all(&["test", "debug", "script-args", "force"])
             )
-            .arg(Arg::with_name("template")
-                .help("Specify a template to use for expression scripts.")
+            .arg(Arg::new("template")
+                .about("Specify a template to use for expression scripts.")
                 .long("template")
-                .short("t")
+                .short('t')
                 .takes_value(true)
                 .requires("expr")
             )
-        .arg(Arg::with_name("list-templates")
-            .help("List the available templates.")
+        .arg(Arg::new("list-templates")
+            .about("List the available templates.")
             .long("list-templates")
             .takes_value(false)
-            .conflicts_with("file-association")
         );
 
     #[cfg(windows)]
     let app = app
         .arg(
-            Arg::with_name("install-file-association")
-                .help("Install a file association so that rust-script executes .ers files.")
+            Arg::new("install-file-association")
+                .about("Install a file association so that rust-script executes .ers files.")
                 .long("install-file-association"),
         )
         .arg(
-            Arg::with_name("uninstall-file-association")
-                .help("Uninstall the file association that makes rust-script execute .ers files.")
+            Arg::new("uninstall-file-association")
+                .about("Uninstall the file association that makes rust-script execute .ers files.")
                 .long("uninstall-file-association"),
         )
         .group(
-            ArgGroup::with_name("file-association")
+            ArgGroup::new("file-association")
                 .args(&["install-file-association", "uninstall-file-association"]),
         );
 
@@ -275,16 +289,9 @@ fn parse_args() -> Args {
             .unwrap_or_default()
     }
 
-    fn yes_or_no(v: Option<&str>) -> Option<bool> {
-        v.map(|v| match v {
-            "yes" => true,
-            "no" => false,
-            _ => unreachable!(),
-        })
-    }
-
     Args {
-        command: owned_vec_string(m.values_of("command")),
+        script: m.value_of("script").map(Into::into),
+        script_args: owned_vec_string(m.values_of("script-args")),
         features: m.value_of("features").map(Into::into),
 
         expr: m.is_present("expr"),
@@ -294,13 +301,13 @@ fn parse_args() -> Args {
         pkg_path: m.value_of("pkg_path").map(Into::into),
         gen_pkg_only: m.is_present("gen_pkg_only"),
         build_only: m.is_present("build_only"),
-        clear_cache: m.is_present("clear_cache"),
+        cargo_output: m.is_present("cargo-output"),
+        clear_cache: m.is_present("clear-cache"),
         debug: m.is_present("debug"),
         dep: owned_vec_string(m.values_of("dep")),
         extern_: owned_vec_string(m.values_of("extern")),
         force: m.is_present("force"),
         unstable_features: owned_vec_string(m.values_of("unstable_features")),
-        use_bincache: yes_or_no(m.value_of("use_bincache")),
         build_kind: BuildKind::from_flags(m.is_present("test"), m.is_present("bench")),
         template: m.value_of("template").map(Into::into),
         list_templates: m.is_present("list-templates"),
@@ -320,28 +327,17 @@ fn main() {
         Ok(code) => {
             std::process::exit(code);
         }
-        Err(ref err) if err.is_human() => {
-            writeln!(stderr, "error: {}", err).unwrap();
-            std::process::exit(1);
-        }
         Err(ref err) => {
-            writeln!(stderr, "internal error: {}", err).unwrap();
+            writeln!(stderr, "error: {}", err).unwrap();
             std::process::exit(1);
         }
     }
 }
 
-fn try_main() -> Result<i32> {
+fn try_main() -> MainResult<i32> {
     let args = parse_args();
     info!("Arguments: {:?}", args);
 
-    if log_enabled!(log::Level::Debug) {
-        let scp = platform::script_cache_path()?;
-        let bcp = platform::binary_cache_path()?;
-        debug!("script-cache path: {:?}", scp);
-        debug!("binary-cache path: {:?}", bcp);
-    }
-
     #[cfg(windows)]
     {
         if args.install_file_association {
@@ -353,18 +349,9 @@ fn try_main() -> Result<i32> {
         }
     }
 
-    /*
-    If we've been asked to clear the cache, do that *now*.  There are two reasons:
-
-    1. Do it *before* we call `decide_action_for` such that this flag *also* acts as a synonym for `--force`.
-    2. Do it *before* we start trying to read the input so that, later on, we can make `<script>` optional, but still supply `--clear-cache`.
-    */
     if args.clear_cache {
         clean_cache(0)?;
-
-        // If we *did not* get a `<script>` argument, that's OK.
-        if args.command.is_empty() {
-            // Just let the user know that we did *actually* run.
+        if args.script.is_none() {
             println!("cargo script cache cleared.");
             return Ok(0);
         }
@@ -381,9 +368,10 @@ fn try_main() -> Result<i32> {
     let script_path: PathBuf;
     let content: String;
 
-    let input = match (args.command[0].clone(), args.expr, args.loop_) {
+    let input = match (args.script.clone().unwrap(), args.expr, args.loop_) {
         (script, false, false) => {
-            let (path, mut file) = find_script(script).ok_or("could not find script")?;
+            let (path, mut file) =
+                find_script(&script).ok_or(format!("could not find script: {}", script))?;
 
             script_name = path
                 .file_stem()
@@ -414,15 +402,21 @@ fn try_main() -> Result<i32> {
     };
     info!("input: {:?}", input);
 
-    /*
-    Sort out the dependencies.  We want to do a few things:
+    // Setup environment variables early so it's available at compilation time of scripts,
+    // to allow e.g. include!(concat!(env!("RUST_SCRIPT_BASE_PATH"), "/script-module.rs"));
+    std::env::set_var(
+        "RUST_SCRIPT_PATH",
+        input.path().unwrap_or_else(|| Path::new("")),
+    );
+    std::env::set_var("RUST_SCRIPT_SAFE_NAME", input.safe_name());
+    std::env::set_var("RUST_SCRIPT_PKG_NAME", input.package_name());
+    std::env::set_var("RUST_SCRIPT_BASE_PATH", input.base_path());
 
-    - Sort them so that they hash consistently.
-    - Check for duplicates.
-    - Expand `pkg` into `pkg=*`.
-    */
-    let deps = {
-        use std::collections::hash_map::Entry::{Occupied, Vacant};
+    // Sort out the dependencies.  We want to do a few things:
+    // - Sort them so that they hash consistently.
+    // - Check for duplicates.
+    // - Expand `pkg` into `pkg=*`.
+    let dependencies_from_args = {
         use std::collections::HashMap;
 
         let mut deps: HashMap<String, String> = HashMap::new();
@@ -442,31 +436,13 @@ fn try_main() -> Result<i32> {
             );
 
             if name == "" {
-                return Err((Blame::Human, "cannot have empty dependency package name").into());
+                return Err(("cannot have empty dependency package name").into());
+            } else if version == "" {
+                return Err(("cannot have empty dependency version").into());
             }
 
-            if version == "" {
-                return Err((Blame::Human, "cannot have empty dependency version").into());
-            }
-
-            match deps.entry(name.into()) {
-                Vacant(ve) => {
-                    ve.insert(version.into());
-                }
-                Occupied(oe) => {
-                    // This is *only* a problem if the versions don't match.  We won't try to do anything clever in terms of upgrading or resolving or anything... exact match or go home.
-                    let existing = oe.get();
-                    if version != existing {
-                        return Err((
-                            Blame::Human,
-                            format!(
-                                "conflicting versions for dependency '{}': '{}', '{}'",
-                                name, existing, version
-                            ),
-                        )
-                            .into());
-                    }
-                }
+            if deps.insert(name.into(), version.into()).is_some() {
+                return Err((format!("duplicated dependency: '{}'", name)).into());
             }
         }
 
@@ -475,11 +451,8 @@ fn try_main() -> Result<i32> {
         deps.sort();
         deps
     };
-    info!("deps: {:?}", deps);
 
-    /*
-    Generate the prelude items, if we need any.  Again, ensure consistent and *valid* sorting.
-    */
+    // Generate the prelude items, if we need any. Ensure consistent and *valid* sorting.
     let prelude_items = {
         let unstable_features = args
             .unstable_features
@@ -497,8 +470,7 @@ fn try_main() -> Result<i32> {
     };
     info!("prelude_items: {:?}", prelude_items);
 
-    // Work out what to do.
-    let action = decide_action_for(&input, deps, prelude_items, &args)?;
+    let action = decide_action_for(&input, dependencies_from_args, prelude_items, &args)?;
     info!("action: {:?}", action);
 
     gen_pkg_and_compile(&input, &action)?;
@@ -515,49 +487,24 @@ fn try_main() -> Result<i32> {
         })
     };
 
-    // Run it!
-    if action.execute {
-        fn hint<F: FnOnce(&mut Command) -> &mut Command>(f: F) -> F {
-            f
-        }
-        let add_env = hint(move |cmd| {
-            cmd.env(
-                "RUST_SCRIPT_PATH",
-                input.path().unwrap_or_else(|| Path::new("")),
-            );
-            cmd.env("RUST_SCRIPT_SAFE_NAME", input.safe_name());
-            cmd.env("RUST_SCRIPT_PKG_NAME", input.package_name());
-            cmd.env("RUST_SCRIPT_BASE_PATH", input.base_path());
-            cmd
-        });
-
+    let exit_code = if action.execute {
         if action.build_kind.can_exec_directly() {
             let exe_path = get_exe_path(action.build_kind, &action.pkg_path)?;
             info!("executing {:?}", exe_path);
-            match {
-                Command::new(exe_path)
-                    .args(&args.command[1..])
-                    .chain_map(add_env)
-                    .status()
-                    .map(|st| st.code().unwrap_or(1))
-            }? {
-                0 => (),
-                n => return Ok(n),
-            }
+            let mut command = Command::new(exe_path);
+            command.args(args.script_args);
+            command.status().map(|st| st.code().unwrap_or(1))?
         } else {
             let cmd_name = action.build_kind.exec_command();
             info!("running `cargo {}`", cmd_name);
             let mut cmd = action.cargo(cmd_name)?;
-            add_env(&mut cmd);
-            match cmd.status().map(|st| st.code().unwrap_or(1))? {
-                0 => (),
-                n => return Ok(n),
-            }
+            cmd.status().map(|st| st.code().unwrap_or(1))?
         }
-    }
+    } else {
+        0
+    };
 
-    // If nothing else failed, I suppose we succeeded.
-    Ok(0)
+    Ok(exit_code)
 }
 
 /**
@@ -565,7 +512,7 @@ Clean up the cache folder.
 
 Looks for all folders whose metadata says they were created at least `max_age` in the past and kills them dead.
 */
-fn clean_cache(max_age: u128) -> Result<()> {
+fn clean_cache(max_age: u128) -> MainResult<()> {
     info!("cleaning cache with max_age: {:?}", max_age);
 
     if max_age == 0 {
@@ -581,7 +528,7 @@ fn clean_cache(max_age: u128) -> Result<()> {
     let cutoff = platform::current_time() - max_age;
     info!("cutoff:     {:>20?} ms", cutoff);
 
-    let cache_dir = platform::script_cache_path()?;
+    let cache_dir = platform::generated_projects_cache_path()?;
     for child in fs::read_dir(cache_dir)? {
         let child = child?;
         let path = child.path();
@@ -635,7 +582,7 @@ Generate and compile a package from the input.
 
 Why take `PackageMetadata`?  To ensure that any information we need to depend on for compilation *first* passes through `decide_action_for` *and* is less likely to not be serialised with the rest of the metadata.
 */
-fn gen_pkg_and_compile(input: &Input, action: &InputAction) -> Result<()> {
+fn gen_pkg_and_compile(input: &Input, action: &InputAction) -> MainResult<()> {
     let pkg_path = &action.pkg_path;
     let meta = &action.metadata;
     let old_meta = action.old_metadata.as_ref();
@@ -708,24 +655,24 @@ fn gen_pkg_and_compile(input: &Input, action: &InputAction) -> Result<()> {
         let mut cmd = cargo(
             "build",
             &*mani_path.to_string_lossy(),
-            action.use_bincache,
             action.use_nightly,
             &meta,
         )?;
 
-        macro_rules! get_status {
-            ($cmd:expr) => {
-                util::suppress_child_output(&mut $cmd)?.status()
-            };
-        }
+        let exit_status = if action.cargo_output {
+            cmd.spawn()?.wait()
+        } else {
+            util::suppress_child_output(&mut cmd)?.status()
+        };
 
-        let compile_err = get_status!(cmd)
-            .map_err(Into::<MainError>::into)
-            .and_then(|st| match st.code() {
-                Some(0) => Ok(()),
-                Some(st) => Err(format!("cargo failed with status {}", st).into()),
-                None => Err("cargo failed".into()),
-            });
+        let compile_err =
+            exit_status
+                .map_err(Into::<MainError>::into)
+                .and_then(|st| match st.code() {
+                    Some(0) => Ok(()),
+                    Some(st) => Err(format!("cargo failed with status {}", st).into()),
+                    None => Err("cargo failed".into()),
+                });
 
         // Drop out now if compilation failed.
         let _ = compile_err?;
@@ -735,19 +682,16 @@ fn gen_pkg_and_compile(input: &Input, action: &InputAction) -> Result<()> {
             input,
             pkg_path,
             &*mani_path.to_string_lossy(),
-            action.use_bincache,
             action.use_nightly,
             &meta,
         )?;
 
-        if action.use_bincache {
-            // Write out the metadata hash to tie this executable to a particular chunk of metadata.  This is to avoid issues with multiple scripts with the same name being compiled to a common target directory.
-            let meta_hash = action.metadata.sha1_hash();
-            info!("writing meta hash: {:?}...", meta_hash);
-            let exe_meta_hash_path = get_meta_hash_path(action.use_bincache, pkg_path)?;
-            let mut f = fs::File::create(&exe_meta_hash_path)?;
-            write!(&mut f, "{}", meta_hash)?;
-        }
+        // Write out the metadata hash to tie this executable to a particular chunk of metadata.  This is to avoid issues with multiple scripts with the same name being compiled to a common target directory.
+        let meta_hash = action.metadata.sha1_hash();
+        info!("writing meta hash: {:?}...", meta_hash);
+        let exe_meta_hash_path = get_meta_hash_path(pkg_path)?;
+        let mut f = fs::File::create(&exe_meta_hash_path)?;
+        write!(&mut f, "{}", meta_hash)?;
     }
 
     // Write out metadata *now*.  Remember that we check the timestamp in the metadata, *not* on the executable.
@@ -770,6 +714,9 @@ struct InputAction {
     /// Compile the input into a fresh executable?
     compile: bool,
 
+    /// Always show cargo output?
+    cargo_output: bool,
+
     /**
     Force Cargo to do a recompile, even if it thinks it doesn't have to.
 
@@ -793,9 +740,6 @@ struct InputAction {
     */
     using_cache: bool,
 
-    /// Use shared binary cache?
-    use_bincache: bool,
-
     use_nightly: bool,
 
     /// The package metadata structure for the current invocation.
@@ -819,11 +763,10 @@ impl InputAction {
         self.pkg_path.join("Cargo.toml")
     }
 
-    fn cargo(&self, cmd: &str) -> Result<Command> {
+    fn cargo(&self, cmd: &str) -> MainResult<Command> {
         cargo(
             cmd,
             &*self.manifest_path().to_string_lossy(),
-            self.use_bincache,
             self.use_nightly,
             &self.metadata,
         )
@@ -881,15 +824,14 @@ fn decide_action_for(
     deps: Vec<(String, String)>,
     prelude: Vec<String>,
     args: &Args,
-) -> Result<InputAction> {
+) -> MainResult<InputAction> {
     let (pkg_path, using_cache) = args
         .pkg_path
         .as_ref()
         .map(|p| (p.into(), false))
         .unwrap_or_else(|| {
             // This can't fail.  Seriously, we're *fucked* if we can't work this out.
-            let cache_path = platform::script_cache_path().unwrap();
-            info!("cache_path: {:?}", cache_path);
+            let cache_path = platform::generated_projects_cache_path().unwrap();
 
             let id = {
                 let deps_iter = deps.iter().map(|&(ref n, ref v)| (n as &str, v as &str));
@@ -904,7 +846,6 @@ fn decide_action_for(
     info!("pkg_path: {:?}", pkg_path);
     info!("using_cache: {:?}", using_cache);
 
-    info!("splitting input...");
     let (mani_str, script_str) = manifest::split_input(input, &deps, &prelude)?;
 
     // Forcibly override some flags based on build kind.
@@ -914,7 +855,6 @@ fn decide_action_for(
         BuildKind::Bench => (false, false, false),
     };
 
-    // Construct input metadata.
     let input_meta = {
         let (path, mtime, template) = match *input {
             Input::File(_, path, _, mtime) => {
@@ -937,15 +877,14 @@ fn decide_action_for(
     };
     info!("input_meta: {:?}", input_meta);
 
-    // Lazy powers, ACTIVATE!
     let mut action = InputAction {
         compile: force,
+        cargo_output: args.cargo_output,
         force_compile: force,
         emit_metadata: true,
         execute: !build_only,
         pkg_path,
         using_cache,
-        use_bincache: args.use_bincache.unwrap_or(using_cache),
         use_nightly: matches!(args.build_kind, BuildKind::Bench),
         metadata: input_meta,
         old_metadata: None,
@@ -980,8 +919,10 @@ fn decide_action_for(
     let cache_meta = match get_pkg_metadata(&action.pkg_path) {
         Ok(meta) => meta,
         Err(err) => {
-            info!("recompiling because: failed to load metadata");
-            debug!("get_pkg_metadata error: {}", err.to_string());
+            info!(
+                "recompiling since failed to load metadata: {}",
+                err.to_string()
+            );
             bail!(compile: true)
         }
     };
@@ -995,9 +936,6 @@ fn decide_action_for(
 
     action.old_metadata = Some(cache_meta);
 
-    /*
-    Next test: does the executable exist at all?
-    */
     let exe_exists = match get_exe_path(action.build_kind, &action.pkg_path) {
         Ok(exe_path) => exe_path.is_file(),
         Err(_) => false,
@@ -1007,31 +945,25 @@ fn decide_action_for(
         bail!(compile: true)
     }
 
-    /*
-    Finally: check to see if `{exe_path}.meta-hash` exists and contains a hash that matches the metadata.  Yes, this is somewhat round-about, but we need to do this to account for cases where Cargo's target directory has been set to a fixed, shared location.
-
-    Note that we *do not* do this if we aren't using the cache.
-    */
-    if action.use_bincache {
-        let exe_meta_hash_path = get_meta_hash_path(action.use_bincache, &action.pkg_path).unwrap();
-        if !exe_meta_hash_path.is_file() {
-            info!("recompiling because: meta hash doesn't exist or isn't a file");
-            bail!(compile: true, force_compile: true)
-        }
-        let exe_meta_hash = {
-            let mut f = fs::File::open(&exe_meta_hash_path)?;
-            let mut s = String::new();
-            f.read_to_string(&mut s)?;
-            s
-        };
-        let meta_hash = action.metadata.sha1_hash();
-        if meta_hash != exe_meta_hash {
-            info!("recompiling because: meta hash doesn't match");
-            bail!(compile: true, force_compile: true)
-        }
+    // Finally  check to see if `{exe_path}.meta-hash` exists and contains a hash that matches the metadata.
+    // We need to do this to account for cases where Cargo's target directory has been set to a fixed, shared location.
+    let exe_meta_hash_path = get_meta_hash_path(&action.pkg_path).unwrap();
+    if !exe_meta_hash_path.is_file() {
+        info!("recompiling because: meta hash doesn't exist or isn't a file");
+        bail!(compile: true, force_compile: true)
+    }
+    let exe_meta_hash = {
+        let mut f = fs::File::open(&exe_meta_hash_path)?;
+        let mut s = String::new();
+        f.read_to_string(&mut s)?;
+        s
+    };
+    let meta_hash = action.metadata.sha1_hash();
+    if meta_hash != exe_meta_hash {
+        info!("recompiling because: meta hash doesn't match");
+        bail!(compile: true, force_compile: true)
     }
 
-    // That's enough; let's just go with it.
     Ok(action)
 }
 
@@ -1040,7 +972,7 @@ Figures out where the output executable for the input should be.
 
 This *requires* that `cargo_target` has already been called on the package.
 */
-fn get_exe_path<P>(build_kind: BuildKind, pkg_path: P) -> Result<PathBuf>
+fn get_exe_path<P>(build_kind: BuildKind, pkg_path: P) -> MainResult<PathBuf>
 where
     P: AsRef<Path>,
 {
@@ -1066,20 +998,17 @@ where
 /**
 Figures out where the `meta-hash` file should be.
 */
-fn get_meta_hash_path<P>(use_bincache: bool, pkg_path: P) -> Result<PathBuf>
+fn get_meta_hash_path<P>(pkg_path: P) -> MainResult<PathBuf>
 where
     P: AsRef<Path>,
 {
-    if !use_bincache {
-        panic!("tried to get meta-hash path when not using binary cache");
-    }
     Ok(pkg_path.as_ref().join("target.meta-hash"))
 }
 
 /**
 Load the package metadata, given the path to the package's cache folder.
 */
-fn get_pkg_metadata<P>(pkg_path: P) -> Result<PackageMetadata>
+fn get_pkg_metadata<P>(pkg_path: P) -> MainResult<PackageMetadata>
 where
     P: AsRef<Path>,
 {
@@ -1110,7 +1039,7 @@ where
 /**
 Save the package metadata, given the path to the package's cache folder.
 */
-fn write_pkg_metadata<P>(pkg_path: P, meta: &PackageMetadata) -> Result<()>
+fn write_pkg_metadata<P>(pkg_path: P, meta: &PackageMetadata) -> MainResult<()>
 where
     P: AsRef<Path>,
 {
@@ -1253,7 +1182,7 @@ impl<'a> Input<'a> {
     /**
     Compute the package ID for the input.  This is used as the name of the cache folder into which the Cargo package will be generated.
     */
-    pub fn compute_id<'dep, DepIt>(&self, deps: DepIt) -> Result<OsString>
+    pub fn compute_id<'dep, DepIt>(&self, deps: DepIt) -> MainResult<OsString>
     where
         DepIt: IntoIterator<Item = (&'dep str, &'dep str)>,
     {
@@ -1340,7 +1269,7 @@ enum FileOverwrite {
 /**
 Overwrite a file if and only if the contents have changed.
 */
-fn overwrite_file<P>(path: P, content: &str, hash: Option<&str>) -> Result<FileOverwrite>
+fn overwrite_file<P>(path: P, content: &str, hash: Option<&str>) -> MainResult<FileOverwrite>
 where
     P: AsRef<Path>,
 {
@@ -1364,10 +1293,9 @@ Constructs a Cargo command that runs on the script package.
 fn cargo(
     cmd_name: &str,
     manifest: &str,
-    use_bincache: bool,
     nightly: bool,
     meta: &PackageMetadata,
-) -> Result<Command> {
+) -> MainResult<Command> {
     let mut cmd = Command::new("cargo");
     if nightly {
         cmd.arg("+nightly");
@@ -1378,14 +1306,12 @@ fn cargo(
         cmd.arg("--color").arg("always");
     }
 
-    if use_bincache {
-        let script_specific_binary_cache = format!(
-            "{}/{}",
-            platform::binary_cache_path()?.display(),
-            meta.sha1_hash()
-        );
-        cmd.env("CARGO_TARGET_DIR", script_specific_binary_cache);
-    }
+    let script_specific_binary_cache = format!(
+        "{}/{}",
+        platform::binary_cache_path()?.display(),
+        meta.sha1_hash()
+    );
+    cmd.env("CARGO_TARGET_DIR", script_specific_binary_cache);
 
     // Block `--release` on `bench`.
     if !meta.debug && cmd_name != "bench" {
@@ -1408,21 +1334,19 @@ fn cargo_target<P>(
     input: &Input,
     pkg_path: P,
     manifest: &str,
-    use_bincache: bool,
     use_nightly: bool,
     meta: &PackageMetadata,
-) -> Result<PathBuf>
+) -> MainResult<PathBuf>
 where
     P: AsRef<Path>,
 {
     trace!(
-        "cargo_target(_, {:?}, {:?}, {:?}, _)",
+        "cargo_target(_, {:?}, {:?}, _)",
         pkg_path.as_ref(),
         manifest,
-        use_bincache
     );
 
-    let exe_path = cargo_target_by_message(input, manifest, use_bincache, use_nightly, meta)?;
+    let exe_path = cargo_target_by_message(input, manifest, use_nightly, meta)?;
 
     trace!(".. exe_path: {:?}", exe_path);
 
@@ -1445,19 +1369,12 @@ where
 fn cargo_target_by_message(
     input: &Input,
     manifest: &str,
-    use_bincache: bool,
     use_nightly: bool,
     meta: &PackageMetadata,
-) -> Result<PathBuf> {
+) -> MainResult<PathBuf> {
     use std::io::{BufRead, BufReader};
 
-    trace!(
-        "cargo_target_by_message(_, {:?}, {:?}, _)",
-        manifest,
-        use_bincache
-    );
-
-    let mut cmd = cargo("build", manifest, use_bincache, use_nightly, meta)?;
+    let mut cmd = cargo("build", manifest, use_nightly, meta)?;
     cmd.arg("--message-format=json");
     cmd.stdout(process::Stdio::piped());
     cmd.stderr(process::Stdio::null());
